@@ -9,7 +9,8 @@ import (
 
 // UART1 on the Circuit Playground Express.
 var (
-	UART1 = UART{
+	UART1  = &_UART1
+	_UART1 = UART{
 		Buffer: NewRingBuffer(),
 		Bus:    sam.SERCOM4_USART,
 		SERCOM: 4,
@@ -17,18 +18,18 @@ var (
 )
 
 func init() {
-	UART1.Interrupt = interrupt.New(sam.IRQ_SERCOM4, UART1.handleInterrupt)
+	UART1.Interrupt = interrupt.New(sam.IRQ_SERCOM4, _UART1.handleInterrupt)
 }
 
 // I2C on the Circuit Playground Express.
 var (
 	// external device
-	I2C0 = I2C{
+	I2C0 = &I2C{
 		Bus:    sam.SERCOM5_I2CM,
 		SERCOM: 5,
 	}
 	// internal device
-	I2C1 = I2C{
+	I2C1 = &I2C{
 		Bus:    sam.SERCOM1_I2CM,
 		SERCOM: 1,
 	}
