@@ -36,10 +36,11 @@ const (
 )
 
 const (
-	LED = D13
+	LED    = D13
+	WS2812 = D8
 )
 
-// UART0 aka USBCDC pins
+// USBCDC pins
 const (
 	USBCDC_DM_PIN = PA24
 	USBCDC_DP_PIN = PA25
